library state_groups;

import 'package:flutter/widgets.dart';

abstract class SyncState<MessageType, T extends StatefulWidget> extends State<T> {

	SyncState(StateGroup<MessageType> group, {this.errorFunction}) {
		_groups.add(group);
	}

	SyncState.Advanced(this._groups, {this.errorFunction});

	List<StateGroup<MessageType>> _groups = <StateGroup<MessageType>>[];
	Function(String)? errorFunction;

	@override
	void initState() {
		for (int i = 0; i < _groups.length; i++) {
			_groups[i].states.add(this);
		}
		super.initState();
	}

	@override
	void dispose() {
		for (int i = 0; i < _groups.length; i++) {
			if (!_groups[i].states.remove(this)) {
				errorFunction!("Could not find $this in state group ${_groups[i]}]");
			}
		}
		super.dispose();
	}

	void update(MessageType? message) {
		if (!shouldUpdate(message)) {
			return;
		}
		WidgetsBinding.instance!.addPostFrameCallback((_) {
			if (mounted) {
				setState(() {});
			}
		});
	}

	bool shouldUpdate(MessageType? message) {
		return true;
	}
}

class StateGroup<MessageType> {

	StateGroup();

	final Set<SyncState<MessageType, dynamic>> states = <SyncState<MessageType, dynamic>>{};

	void notifyAll([MessageType? message]) {
		for (final SyncState<MessageType, dynamic> state in states) {
			state.update(message);
		}
	}

	@override
	String toString() {
		return "State Group ($states)";
	}
}

class StateGroupCollection<MessageType> {

	StateGroupCollection(this.collections);

	final List<StateGroup<MessageType>> collections;

	void notifyAll([MessageType? message]) {
		for (int i = 0; i < collections.length; i++) {
			collections[i].notifyAll(message);
		}
	}

	@override
	String toString() {
		return "State Group Collection: ($collections)";
	}
}
